<?php

namespace InspireLabs\WooInpost\admin;

use InspireLabs\WooInpost\EasyPack;
use Exception;
use function False\true;

/**
 * EasyPack Dispatch Orders
 *
 * @author      WPDesk
 * @category    Admin
 * @package     EasyPack/Admin
 * @version     2.1.0
 */

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

if (!class_exists('EasyPack_Dispatch_Orders')) :

    /**
     * EasyPack_Dispatch_Orders
     */
    class EasyPack_Dispatch_Orders
    {

        protected static $instance;

        public function __construct()
        {
            add_action('woocommerce_register_post_type',
              [$this, 'register_post_types'], 5);

            add_action('manage_dispatch_order_posts_custom_column',
              [$this, 'manage_dispatch_order_posts_custom_column'], 10, 2);

            add_filter('manage_edit-dispatch_order_columns',
              [$this, 'manage_edit_dispatch_order_columns']);

            add_filter('bulk_actions-' . 'edit-dispatch_order',
              '__return_empty_array');

            add_action('admin_notices', [$this, 'admin_notices']);
        }

        public static function EasyPack_Dispatch_Orders()
        {
            if (self::$instance === null) {
                self::$instance = new self();
            }
            return self::$instance;
        }

        public function create_dispatch_order($dispatch_point)
        {
            $parcels    = $_POST['easypack_parcel'];
            $parcel_ids = [];
            foreach ($parcels as $parcel) {
                $parcel_exploded = explode('.', $parcel);
                if ($parcel_exploded[0] == 'easypack') {
                    $parcel_ids[] = $parcel_exploded[2];
                }
            }
            $args = ['parcel_ids' => $parcel_ids];

            try {
                $response = EasyPack_API()->dispatch_order($dispatch_point,
                  $args);
                return $response['id'];
            } catch (Exception $e) {
                throw $e;
            }
        }

        public function create_manifest(
          $easypack_dispatch_point,
          $easypack_dispatch_number
        ) {


            $ret = [];

            $easypack_parcel_ids = $_POST['easypack_parcel'];

            $all_parcels = ['crossborder' => [], 'easypack' => []];

            foreach ($easypack_parcel_ids as $easypack_parcel_id) {
                $order_parcel = explode('.', $easypack_parcel_id);
                $api          = $order_parcel[0];
                $order_id     = $order_parcel[1];
                $parcel_id    = $order_parcel[2];

                $all_parcels[$api][] = [
                  'parcel_id' => $parcel_id,
                  'order_id'  => $order_id,
                ];
            }
            foreach ($all_parcels as $api => $api_parcels) {
                if (count($api_parcels)) {
                    try {

                        $post               = [
                          'post_type'   => 'dispatch_order',
                          'post_status' => 'publish',
                        ];
                        $post['post_title'] = $api . ' - ' . date_i18n(get_option('date_format') . ' ' . get_option('time_format'),
                            current_time('timestamp'));
                        $post_id            = wp_insert_post($post);
                        $post               = get_post($post_id);
                        $post->post_title   = $post_id . ' - ' . $post->post_title;
                        wp_update_post($post);
                        update_post_meta($post_id, '_parcels', $api_parcels);


                        $sender = get_option('easypack_sender_email');
                        if (get_option('easypack_sender_company_name')) {
                            $sender .= "\n" . get_option('easypack_sender_company_name');
                        }
                        $sender .= "\n" . get_option('easypack_sender_first_name') . ' ' . get_option('easypack_sender_last_name');
                        $sender .= "\n" . get_option('easypack_sender_street') . ' ' . get_option('easypack_sender_building_no');
                        if (get_option('easypack_sender_flat_no')) {
                            $sender .= "/" . get_option('easypack_sender_flat_no');
                        }
                        $sender .= "\n" . get_option('easypack_sender_post_code') . ' ' . get_option('easypack_sender_city');
                        $sender .= "\nTel.:" . get_option('easypack_sender_phone');
                        update_post_meta($post_id, '_sender', $sender);

                        update_post_meta($post_id, '_dispatch_number',
                          $easypack_dispatch_number);

                        if ($api == 'easypack') {
                            $dispatch_point = EasyPack_API()->dispatch_point($easypack_dispatch_point);
                            update_post_meta($post_id, '_dispatch_point',
                              $dispatch_point);
                        }
                        update_post_meta($post_id, '_api', $api);

                        update_post_meta($post_id, '_parcel_ids', $api_parcels);

                        $ret[] = ['post_id' => $post_id, 'api' => $api];

                    } catch (Exception $e) {
                        throw $e;
                    }
                }
            }

            foreach ($easypack_parcel_ids as $easypack_parcel_id) {

                $order_parcel = explode('.', $easypack_parcel_id);
                $api          = $order_parcel[0];
                $order_id     = $order_parcel[1];
                $parcel_id    = $order_parcel[2];

                $easypack_parcels = get_post_meta($order_id,
                  '_easypack_parcels', true);

                foreach ($easypack_parcels as $key => $easypack_parcel) {
                    if ($api == 'crossborder') {
                        if ($easypack_parcel['crossborder_data']['id'] == $parcel_id) {
                            $easypack_parcels[$key]['dispatch_order'] = $post_id;
                        }
                    } else {
                        if ($easypack_parcel['easypack_data']['id'] == $parcel_id) {
                            $easypack_parcels[$key]['dispatch_order'] = $post_id;
                        }
                    }
                }
                update_post_meta($order_id, '_easypack_parcels',
                  $easypack_parcels);

                $dispatched = true;
                foreach ($easypack_parcels as $key => $easypack_parcel) {
                    if (empty($easypack_parcels[$key]['dispatch_order'])) {
                        $dispatched = false;
                    }
                }
                if ($dispatched) {
                    update_post_meta($order_id, '_easypack_dispatched', '1');
                }

            }

            return $ret;

        }

        public function get_crossborder_manifest($post_id)
        {
            $manifest = get_post_meta($post_id, '_manifest');
            header('Location: ' . $manifest[0]['pdf_url']);
        }

        public function register_post_types()
        {
            $args =
              [
                'labels'              => [
                  'name'               => __('Shipments', 'flexible-shipping'),
                  'singular_name'      => __('Shipment', 'flexible-shipping'),
                  'menu_name'          => __('Shipments', 'flexible-shipping'),
                  'parent_item_colon'  => '',
                  'all_items'          => __('Shipments', 'flexible-shipping'),
                  'view_item'          => __('View Shipments',
                    'flexible-shipping'),
                  'add_new_item'       => __('Add new Shipment',
                    'flexible-shipping'),
                  'add_new'            => __('Add new Shipment',
                    'flexible-shipping'),
                  'edit_item'          => __('Edit Shipment',
                    'flexible-shipping'),
                  'update_item'        => __('Save Shipment',
                    'flexible-shipping'),
                  'search_items'       => __('Search Shipments',
                    'flexible-shipping'),
                  'not_found'          => __('Shipment not found',
                    'flexible-shipping'),
                  'not_found_in_trash' => __('Shipment not found in trash',
                    'flexible-shipping'),
                ],
                'description'         => __('Shipments.', 'flexible-shipping'),
                'public'              => false,
                'show_ui'             => true,
                'capability_type' => 'post',
                'capabilities' => array(
	                'create_posts' => 'do_not_allow', // Removes support for the "Add New" function, including Super Admin's
                ),

                'publicly_queryable'  => false,
                'exclude_from_search' => true,
                'hierarchical'        => false,
                'query_var'           => true,
                'supports'            => ['title'],
                'has_archive'         => false,
                'show_in_nav_menus'   => true,
                'menu_icon'           => 'dashicons-upload',
              ];


            register_post_type('dispatch_order', $args);
        }

        //public function manage_posts_custom_column

        public function manage_edit_dispatch_order_columns($columns)
        {
            unset($columns['date']);
            unset($columns['cb']);
            $columns['created'] = __('Created', EasyPack::$text_domain);
            //$columns['status'] = __( 'Status', EasyPack::$text_domain );
            $columns['action'] = __('Action', EasyPack::$text_domain);
            return $columns;
        }

        function manage_dispatch_order_posts_custom_column($column, $post_id)
        {
            global $post;

            $api    = get_post_meta($post_id, '_api', true);
            $status = get_post_meta($post_id, '_status', true);

            switch ($column) {
                case 'created' :
                    echo $post->post_date;
                    break;
                case 'status' :
                    if ($status == 'cancelled') {
                        _e('Cancelled', EasyPack::$text_domain);
                    }
                    break;
                case 'action' :
                    if ($status != 'cancelled') {
                        $url  = site_url('?easypack_download=1&easypack_manifest=' . $post_id);
                        $link = '<a href="' . $url . '" target="blank" class="button-primary">' . __('Print',
                            EasyPack::$text_domain) . '</a>';
                        echo $link;
                        /*echo ' ';
              $url = admin_url( 'edit.php?post_type=dispatch_order&cancel=' . $post_id );
              $link = '<a href="' . $url . '" class="button easypack_cancel_dispatch_order">' . __( 'Cancel', EasyPack::$text_domain ) . '</a>';
              echo $link;*/
                    }
                    break;
                default        :
                    break;
            }
        }

        function cancel_dispatch_order($post)
        {
            $order_parcel_ids = get_post_meta($post->ID, '_parcel_ids', true);
            $api              = get_post_meta($post->ID, '_api', true);

            foreach ($order_parcel_ids as $order_parcel_id) {
                $order_id  = $order_parcel_id['order_id'];
                $parcel_id = $order_parcel_id['parcel_id'];

                $easypack_parcels = get_post_meta($order_id,
                  '_easypack_parcels', true);
                foreach ($easypack_parcels as $key => $easypack_parcel) {
                    if ($api == 'crossborder') {
                        if ($easypack_parcel['crossborder_data']['id'] == $parcel_id) {
                            unset($easypack_parcels[$key]['dispatch_order']);
                        }
                    } else {
                        if ($easypack_parcel['easypack_data']['id'] == $parcel_id) {
                            unset($easypack_parcels[$key]['dispatch_order']);
                        }
                    }
                }
                update_post_meta($order_id, '_easypack_parcels',
                  $easypack_parcels);

                delete_post_meta($order_id, '_easypack_dispatched');

            }
            update_post_meta($post->ID, '_status', 'cancelled');
        }

        function admin_notices()
        {
            global $pagenow;
            if ($pagenow == 'edit.php' && isset($_GET['post_type']) && $_GET['post_type'] == 'dispatch_order' && isset($_GET['cancel'])) {
                $post = get_post($_GET['cancel']);

                $dispatch_order = EasyPack()
                  ->get_courier_pickup_service()
                  ->get_dispatch_order((int)$_GET['cancel']);

                $status = $dispatch_order->getStatus();


                if ($post) {
                    if (get_post_meta($post->ID, '_status') == 'cancelled') {
                        ?>
                        <div class="error">
                            <p><?php echo sprintf(__('Dispatch order %d already cancelled.',
                                  EasyPack::$text_domain), $post->ID); ?></p>
                        </div>
                        <?php
                    } else {
                        $this->cancel_dispatch_order($post);
                        ?>
                        <div class="updated">
                            <p><?php echo sprintf(__('Dispatch order %d cancelled.',
                                  EasyPack::$text_domain), $post->ID); ?></p>
                        </div>
                        <?php
                    }
                }
            }
        }

    }

endif;
